using System.Text;

namespace Sharp.Augeas;

public static class TreeExtensions
{
    /// <summary>
    /// Builds a typed tree with the <see cref="augeasPath"/>
    /// </summary>
    /// <param name="augeas"></param>
    /// <param name="superNodeName">Super node name.</param>
    /// <param name="augeasPath">Augeas path.</param>
    /// <returns></returns>
    public static SuperNode GetTree(this Augeas augeas, string superNodeName, string augeasPath)
    {
        var basePath = augeasPath.Replace("*", "");
        var tree = augeas.GetDictionaryTree(augeasPath);
        Dictionary<string, Directive> addedDirectives = new Dictionary<string, Directive>();
        Dictionary<string, SuperNode> addedSuperNodes = new Dictionary<string, SuperNode>();
        addedSuperNodes.Add(superNodeName, new SuperNode(augeas, augeasPath, superNodeName));

        foreach (var treeKey in tree.Keys)
        {
            var hierarchy = treeKey.Split('/');
            var firstElement = hierarchy[0];
            // Parent
            if (hierarchy.Length <= 1)
            {
                var rFullPath = $"{basePath}{firstElement}";
                if (firstElement.StartsWith("directive"))
                {
                    Directive directive = new Directive(augeas, firstElement, rFullPath, tree[firstElement]);
                    directive.Parent = addedSuperNodes[superNodeName];
                    addedSuperNodes[superNodeName].AddDirective(directive);
                    addedDirectives.Add(treeKey, directive);
                    continue;
                }
                // If is argument
                if (firstElement.StartsWith("arg"))
                {
                    Argument argument = new Argument(augeas, firstElement, rFullPath, tree[firstElement]);
                    argument.Parent = addedSuperNodes[superNodeName];
                    addedSuperNodes[superNodeName].AddArgument(argument);
                    continue;
                }
                if (IsSuperNodeCandidate(firstElement) && !addedSuperNodes.ContainsKey(firstElement))
                {
                    var superNode = new SuperNode(augeas, rFullPath, firstElement);
                    superNode.Parent = addedSuperNodes[superNodeName];
                    addedSuperNodes.Add(firstElement, superNode);
                }
                continue;
            }
            // handle children
            for (var i = 1; i < hierarchy.Length; i++)
            {
                var childKey = hierarchy[i];
                var parentPathBuilder = new StringBuilder();

                for (int j = 0; j < i; j++) parentPathBuilder.Append($"{hierarchy[j]}/");

                var parentPath = parentPathBuilder.ToString();
                parentPath = parentPath.Trim('/');
                var fullPath = $"{parentPath}/{childKey}";
                var rFullPath = $"{basePath}{fullPath}";
                var childValue = tree[fullPath];
                
                if (IsSuperNodeCandidate(firstElement) && !addedSuperNodes.ContainsKey(firstElement))
                {
                    var superNode = new SuperNode(augeas, rFullPath, firstElement);
                    superNode.Parent = addedSuperNodes[superNodeName];
                    addedSuperNodes.Add(firstElement, superNode);
                }
                if (childKey.StartsWith("directive"))
                {
                    Directive directive = new Directive(augeas, childKey, rFullPath, childValue);
                    if (addedDirectives.ContainsKey(fullPath)) continue;
                    addedDirectives.Add(fullPath, directive);
                    if (addedSuperNodes.ContainsKey(parentPath))
                        addedSuperNodes[parentPath].AddDirective(directive);

                    continue;
                }
                if (childKey.StartsWith("arg"))
                {
                    Argument argument = new Argument(augeas, childKey, rFullPath, childValue);

                    if (addedDirectives.ContainsKey(parentPath))
                    {
                        addedDirectives[parentPath].AddArgument(argument);
                        continue;
                    }

                    if (addedSuperNodes.ContainsKey(parentPath))
                        addedSuperNodes[parentPath].AddArgument(argument);
                }
            }
        }

        var superNodes = addedSuperNodes.Keys;

        foreach (var key in superNodes)
        {
            if (key != superNodeName)
            {
                addedSuperNodes[superNodeName].AddSuperNode(addedSuperNodes[key]);
            }
        }

        return addedSuperNodes[superNodeName];
    }
    
    
    private static bool IsSuperNodeCandidate(string key)
    {
        return !key.StartsWith("arg") && !key.StartsWith("directive");
    }

}